unit bzip2_interface;
{<
  bzip2_interface contains interface utility for easy using of BZIP2 functionality.

  Full functional demo.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(Apr 2014)
  @lastmod(Apr 2014)
}

interface

uses Sysutils, Classes;

function CompressStream(Source, Dest: TStream): Int64;
function DeCompressStream(Source, Dest: TStream): Int64;

function CompressFile(SourceFile, DestFile: string): Int64;
function DeCompressFile(SourceFile, DestFile: string): Int64;

function CompressString(SourceStr: string; SaveAsFile: string = ''): string;
function DeCompressString(SourceStr: string; LoadFromFile: string = ''): string;

implementation

uses bzip2_main;

function CompressStream(Source, Dest: TStream): Int64;
var
  Comp: TBZCompressionStream;
begin
  Comp:=TBZCompressionStream.Create(bs9, Dest);
  try
    Comp.CopyFrom(Source, 0);
  finally
    Comp.Free;
  end;
  Result:=Dest.Size;
end;

function DeCompressStream(Source, Dest: TStream): Int64;
const
  BufferSize = 65536;
var
  Comp: TBZDecompressionStream;
  Count: Integer;
  Buffer: TBytes;
begin
  Comp:=TBZDecompressionStream.Create(Source);
  try
    SetLength(Buffer, BufferSize);
    repeat
      Count:=Comp.Read(Buffer[0], BufferSize);
      if Count <> 0 then
        Dest.WriteBuffer(Buffer[0], Count)
      else
        Break;
    until false;
    Result:=Dest.Size;
  finally
    Comp.Free;
  end;
end;

function CompressFile(SourceFile, DestFile: string): Int64;
var
  Source, Dest: TFileStream;
begin
  Source:=TFileStream.Create(SourceFile, fmOpenRead + fmShareDenyNone);
  try
    Dest:=TFileStream.Create(DestFile, fmCreate);
    try
      Result:=CompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

function DeCompressFile(SourceFile, DestFile: string): Int64;
var
  Source, Dest: TFileStream;
begin
  Source:=TFileStream.Create(SourceFile, fmOpenRead + fmShareDenyWrite);
  try
    Dest:=TFileStream.Create(DestFile, fmCreate);
    try
      Result:=DeCompressStream(Source, Dest);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

function CompressString(SourceStr: string; SaveAsFile: string = ''): string;
var
  Source, Dest: TStringStream;
begin
  Source:=TStringStream.Create(SourceStr);
  try
    Dest:=TStringStream.Create('');
    try
      CompressStream(Source, Dest);
      Dest.Seek(0, soFromBeginning);
      Result:=Dest.DataString;
      if SaveAsFile<>'' then
        Dest.SaveToFile(SaveAsFile);
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

function DeCompressString(SourceStr: string; LoadFromFile: string = ''): string;
var
  Source, Dest: TStringStream;
begin
  Source:=TStringStream.Create(SourceStr);
  if LoadFromFile<>'' then
    Source.LoadFromFile(LoadFromFile);
  try
    Dest:=TStringStream.Create;
    try
      DeCompressStream(Source, Dest);
      Dest.Seek(0, soFromBeginning);
      Result:=Dest.DataString;
    finally
      Dest.Free;
    end;
  finally
    Source.Free;
  end;
end;

end.
